
	#include "stm32f4xx.h"
	#include "stm32f4xx_rcc.h"
	#include "stm32f4xx_gpio.h"
	#include "stm32f4xx_tim.h"
	#include "stm32f4xx_dma.h"
	#include "stm32f4xx_spi.h"
	#include "misc.h"

	#include "Font.h"

	//					              I-B-G-R-I-B-G-R-
	#define Pixel_Black 			0b0000000000000000
	#define Pixel_Blue				0b0000000000100000
	#define Pixel_Red 				0b0000000000000010
	#define Pixel_Magenta 			0b0000000000100010
	#define Pixel_Green  			0b0000000000001000
	#define Pixel_Cyan 				0b0000000000101000
	#define Pixel_Yellow 			0b0000000000001010
	#define Pixel_White 			0b0000000000101010
	#define Pixel_Bright_Blue		0b0000000010100000
	#define Pixel_Bright_Red 		0b0000000010000010
	#define Pixel_Bright_Magenta 	0b0000000010100010
	#define Pixel_Bright_Green  	0b0000000010001000
	#define Pixel_Bright_Cyan 		0b0000000010101000
	#define Pixel_Bright_Yellow 	0b0000000010001010
	#define Pixel_Bright_White 		0b0000000010101010

	#define Fon_Black 				0b0000000000000000
	#define Fon_Blue				0b0010000000000000
	#define Fon_Red 				0b0000001000000000
	#define Fon_Magenta 			0b0010001000000000
	#define Fon_Green  				0b0000100000000000
	#define Fon_Cyan 				0b0010100000000000
	#define Fon_Yellow 				0b0000101000000000
	#define Fon_White 				0b0010101000000000
	#define Fon_Bright_Blue			0b1010000000000000
	#define Fon_Bright_Red 			0b1000001000000000
	#define Fon_Bright_Magenta 		0b1010001000000000
	#define Fon_Bright_Green  		0b1000100000000000
	#define Fon_Bright_Cyan 		0b1010100000000000
	#define Fon_Bright_Yellow 		0b1000101000000000
	#define Fon_Bright_White 		0b1010101000000000





	#define	VID_HSIZE		50			// Роздільна здатність по горизонталі  // Horizontal resolution (in bytes)
	#define	VID_VSIZE		300			// Роздільна здатність по вертикалі (в рядках) // Vertical resolution (in lines)
	#define VTOTAL			51			// Загальна кількість байтів для надсилання через SPI /* Total bytes to send through SPI for one horizontal line */

	uint16_t fb[VID_VSIZE][VTOTAL];		// Буфер кадрів
	uint32_t vdraw = 0;					// Використовується для збільшення vline кожні 2 намальовані лінії
	uint16_t vline = 0;					// Поточна лінія, що малюється
	uint32_t vflag = 0;					// Коли 1, запит SPI DMA може відображатися на екрані


	uint32_t i = 0;
	uint32_t C = 0;





	//************************************************ *****************************
	// Переривання по співпадінню таймера 3.
	//************************************************ *****************************
	void TIM3_IRQHandler(void)
	{
		if(vflag==1)
		DMA2_Stream3->CR |= DMA_SxCR_EN;		// Встановлення біта EN, для увімкнення DMA.
	  	TIM3->SR &= ~TIM_IT_CC2;				// Скидаєм прапор другого каналу співпадіня таймера 3, інакше знову будемо тут.
	}


	//*****************************************************************************
	// Переривання по співпадінню таймера 4.
	//************************************************ *****************************
	void TIM4_IRQHandler(void)
	{
		vflag = 1;
	  	TIM4->SR &= ~TIM_IT_CC2;				// Скидаєм прапор другого каналу співпадіня таймера 4, інакше знову будемо тут.
	}


	//******************************************************************************
	// Переривання від DMA після закінчення передачі блоку відеоданих.
	//******************************************************************************
	void  DMA2_Stream3_IRQHandler(void)
	{
		DMA2->LIFCR = DMA_LIFCR_CTCIF3;			// Скидаєм прапор переривання DMA, інакше знову будемо тут.
		DMA2_Stream3->CR &= ~DMA_SxCR_EN;		// Вимкнення DMA.
		DMA2_Stream3->NDTR = VTOTAL;			// Розмір буфера даних що буде переданий.

		vdraw++;

		if (vdraw == 2)							// Кожна лінія малюється 2 рази.
		{
			vdraw = 0;

			vline++;

			if (vline == VID_VSIZE)
			{
				vdraw = vline = vflag = 0;
				DMA2_Stream3->M0AR = (uint32_t) &fb[0][0];
			}
			else
			DMA2_Stream3->M0AR += 102;
		}
	}




	void TIMER_Configuration(void)
	{
		GPIO_InitTypeDef			GPIO_InitStructure;
		TIM_TimeBaseInitTypeDef		TIM_TimeBaseStructure;
		TIM_OCInitTypeDef			TIM_OCInitStructure;
		NVIC_InitTypeDef			NVIC_InitStructure;

		// Налаштування пінів PB4 і PB6.
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;							// Пін керується переферією.
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;						// Швидкість роботи піна.
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;							// Двотактний вихід.
		GPIO_Init(GPIOB, &GPIO_InitStructure);									// Примінити для порта B.

		// Налаштування PB4 і PB6 для альтернативної функції TIM3 і TIM4.
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_TIM3);
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_TIM4);

		// Налаштування роботи TIM3.
		TIM_TimeBaseStructure.TIM_Prescaler = 0;								// Робота без вхідного дільника.
		TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;				// Режим рахунку вгору.
		TIM_TimeBaseStructure.TIM_Period = 2112;								// Після числа 2112 таймер переповниться. (VGA Whole line 1056*2).
		TIM_TimeBaseStructure.TIM_ClockDivision = 0;							//
		TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;						// Лічильник переповнень.
		TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);							// Примінити для таймера 3.

		// Налаштування виходів для TIM3.
		TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;						// Режим виходу PWM2.
		TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;			// Стан виходу увімкнений.
		TIM_OCInitStructure.TIM_Pulse = 256;									// Тривалість імпульсу.	(VGA Sync pulse	128*2)
		TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;				// При співпадінні пін у низькому рівні.
		TIM_OC1Init(TIM3, &TIM_OCInitStructure);								// Примінити для таймера 3.

		TIM3->CCR2=363;															// Число при якому відбудеться переривання. (VGA Sync pulse+Back porch 128+88*2)

		TIM_CtrlPWMOutputs(TIM3, ENABLE);										// Увімкнення виходу таймера.

		// Вибір таймера 3 як Master
		TIM_SelectMasterSlaveMode(TIM3, TIM_MasterSlaveMode_Enable);			// Увімкнення події від тригера.
		TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);					// Увімкнення тригера таймера


		// Вибір таймера 4 як Slave
		TIM_SelectSlaveMode(TIM4, TIM_SlaveMode_Gated);
		TIM_SelectInputTrigger(TIM4, TIM_TS_ITR2);

		// Налаштування роботи TIM4.
		TIM_TimeBaseStructure.TIM_Prescaler = 0;								// Робота без вхідного дільника.
		TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;				// Режим рахунку вгору.
		TIM_TimeBaseStructure.TIM_Period = 628;									// Після числа 628 таймер переповниться. (VGA Whole frame 628).
		TIM_TimeBaseStructure.TIM_ClockDivision = 0;							//
		TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;						// Лічильник переповнень.
		TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);							// Примінити для таймера 4.

		// Налаштування виходів для TIM4.
		TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;						// Режим виходу PWM2.
		TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;			// Стан виходу увімкнений.
		TIM_OCInitStructure.TIM_Pulse = 4;										// Тривалість імпульсу.	(VGA Sync pulse	4)
		TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;				// При співпадінні пін у низькому рівні.
		TIM_OC1Init(TIM4, &TIM_OCInitStructure);								// Примінити для таймера 4.

		TIM4->CCR2=27;															// Число при якому відбудеться переривання. (VGA Sync pulse+Back porch 4+23)

		TIM_CtrlPWMOutputs(TIM4, ENABLE);										// Увімкнення виходу таймера.

		// Налаштування преорітету переривання TIM3.
		NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;							// Переривання TIM3_IRQn.
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;				// Приорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;						// Субприорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;							// Увімкнути переривання.
		NVIC_Init(&NVIC_InitStructure);											// Примінити для TIM3.

		// Налаштування преорітету переривання TIM4.
		NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQn;							// Переривання TIM4_IRQn.
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;				// Приорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;						// Субприорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;							// Увімкнути переривання.
		NVIC_Init(&NVIC_InitStructure);											// Примінити для TIM4.

		TIM_ITConfig(TIM3, TIM_IT_CC2, ENABLE);									// Увімкнення переривання по співпадінню канала 2 таймера 3.
		TIM_ITConfig(TIM4, TIM_IT_CC2, ENABLE);									// Увімкнення переривання по співпадінню канала 2 таймера 4.

		TIM_Cmd(TIM3, ENABLE);													// Увімкнення  таймера 3.
		TIM_Cmd(TIM4, ENABLE);													// Увімкнення  таймера 4.
	}


	void SPI_Configuration(void)
	{
		GPIO_InitTypeDef		GPIO_InitStructure;
		DMA_InitTypeDef			DMA_InitStructure;
		SPI_InitTypeDef			SPI_InitStructure;
		NVIC_InitTypeDef 		NVIC_InitStructure;

		// Налаштування пінів PB3 і PB5.
		GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_3 | GPIO_Pin_5;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;									// Пін керується переферією.
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;								// Швидкість роботи піна.
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;									// Двотактний вихід.
		GPIO_Init(GPIOB, &GPIO_InitStructure);											// Примінити для порта B.

		// Налаштування PB3 і PB5 для альтернативної функції SPI1.
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_SPI1);
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_SPI1);

		SPI_I2S_DeInit(SPI1);															// Скидання налаштувань SPI.
		SPI_Cmd(SPI1, DISABLE);															// Вимкнення SPI.
		DMA_DeInit(DMA2_Stream3);														// Скидання налаштувань DMA.

		DMA_StructInit(&DMA_InitStructure);
		DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;					// Адреса периферійного пристрою.
		DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t) &fb[0][0];					// Адреса пам`яті.
		DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;							// Напрямок передачі, з пам`яті - у периферію.
		DMA_InitStructure.DMA_BufferSize = VTOTAL;										// Розмір буфера даних.
		DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;				// Не інкрементувати адресу на дані у периферії.
		DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;							// Інкрементувати адресу на дані у пам`яті.
		DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;		// Розмір даних периферії 16 біт.
		DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;				// Розмір даних пам`яті 16 біт.
		DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;									// Режим роботи каналу DMA.
		DMA_InitStructure.DMA_Priority = DMA_Priority_High;								// Високий приорітет каналу DMA.
		DMA_InitStructure.DMA_Channel = DMA_Channel_3;									// Номер каналу DMA.
		DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;							// Режим FIFO вимкнений.
		DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;						//
		DMA_InitStructure.DMA_PeripheralBurst = DMA_MemoryBurst_Single;					//
		DMA_Init(DMA2_Stream3, &DMA_InitStructure);										// Примінити для 3 каналу DMA.

		SPI_StructInit(&SPI_InitStructure);
		SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Tx;						// Налаштовання ТХ на передачу.
		SPI_InitStructure.SPI_Mode = SPI_Mode_Master;									// Режим - мастер.
		SPI_InitStructure.SPI_DataSize = SPI_DataSize_16b;								// Розмір даних 16 біт.
		SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;										// Полярність тактового сигналу.
		SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;									// Фаза тактового сигналу.
		SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;										// Керування піном NSS програмно.
		SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;				// Поділювач тактового сигналу SPI 2.
		SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;								// Першим відправляється старший біт.
		SPI_InitStructure.SPI_CRCPolynomial = 7;										//
		SPI_Init(SPI1, &SPI_InitStructure);												// Примінити для SPI1.

		SPI_CalculateCRC(SPI1, DISABLE);												// Не розраховувати контрольну суму.

		NVIC_InitStructure.NVIC_IRQChannel = DMA2_Stream3_IRQn;							// Переривання DMA2_Stream3.
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;						// Приорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;								// Субприорітет переривання.
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;									// Увімкнути переривання.
		NVIC_Init(&NVIC_InitStructure);													// Примінити для DMA.

		DMA2_Stream3->CR &= ~DMA_SxCR_EN;												// Очищення біта EN, для вимкнення потоку.
		DMA2_Stream3->NDTR = VTOTAL;													// Розмір буфера даних для передачі.
		DMA2_Stream3->M0AR = (uint32_t) &fb[0][0];										// Початок буфера даних.

		SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Tx, ENABLE);								// Дозволити перериванню Tx генерувати запити DMA.
		SPI_Cmd(SPI1, ENABLE);															// Увімкнення SPI.

		DMA_ITConfig(DMA2_Stream3, DMA_IT_TC, ENABLE);									// Увімкнення переривання від DMA після закінчення передачі блоку.
	}


	void Clear_Memory_Screen(void)
	{
		uint16_t x, y;

		for (y = 0; y < VID_VSIZE; y++)
		{
			for (x = 0; x < VTOTAL; x++)
			{
				fb[y][x] = 0;
			}
		}
	}


	void vidInit(void)
	{
		SPI_Configuration();
		TIMER_Configuration();
		Clear_Memory_Screen();
	}




	//===============================================================
	// Зафарбовування екрана по координатах
	//===============================================================
	void Fill_With_Color(uint16_t Yb, uint16_t Ye, uint8_t Xb, uint8_t Xe, uint16_t Color_F, uint16_t Color_P)
	{
		uint8_t x;
		uint16_t y;

		for (x = Xb; x < Xe; x++)
		{
			for (y = Yb; y < Ye; y++)
			{
				fb[y][x] = Color_F;
				fb[y][x] |= Color_P;
			}
		}
	}



	//===============================================================
	// Малюєм символ 32X48
	//===============================================================
	void Big_Symbol(uint16_t y, uint8_t x, uint8_t Dta)
	{
		uint8_t i;
		uint8_t c;
		uint8_t h;
		uint8_t Data;
		uint16_t Temp;
		uint8_t m;

		m=0;
		for (i=0;i<48;i++)
		{
			for (h=0;h<4;h++)
			{
				Data=Big_Font[Dta][m];

				for(c = 0; c < 8; c++)
				{
					Temp <<= 2;
					if (Data&0x80)
					Temp  |= (1<<0);
					Data <<= 1;
				}

				m++;

				fb[i+y][h+x] &= 0xAAAA;
				fb[i+y][h+x] |= Temp;
			}
		}
	}




	//===============================================================
	// Малюєм символ 16X20
	//===============================================================
	void Average_Symbol(uint16_t y, uint8_t x, uint8_t Dta)
	{
		uint8_t i;
		uint8_t c;
		uint8_t h;
		uint8_t Data;
		uint16_t Temp;
		uint8_t m;

		m=0;
		for (i=0;i<20;i++)
		{
			for (h=0;h<2;h++)
			{
				Data=Average_Font[Dta][m];

				for(c = 0; c < 8; c++)
				{
					Temp <<= 2;
					if (Data&0x80)
					Temp  |= (1<<0);
					Data <<= 1;
				}

				m++;

				fb[i+y][h+x] &= 0xAAAA;
				fb[i+y][h+x] |= Temp;
			}
		}
	}


	//===============================================================
	// Малюєм символ 8X8
	//===============================================================
	void Small_Symbol(uint16_t y, uint8_t x, uint8_t Dta)
	{
		uint8_t i;
		uint8_t c;
		uint8_t Data;
		uint16_t Temp;

		for (i=0;i<8;i++)
		{
			Data=Small_font[Dta][i];

			for(c = 0; c < 8; c++)
			{
				Temp <<= 2;
				if (Data&0x80)
				Temp  |= (1<<0);
				Data <<= 1;
			}

			fb[i+y][x] &= 0xAAAA;
			fb[i+y][x] |= Temp;
		}
	}



	//===============================================================
	// Малюєм піксель
	//===============================================================
	void Put_Pixel(uint16_t x, uint16_t y)
	{
		uint16_t  w, r, Data, Temp;
		uint8_t	  m;
		uint8_t	  c;

		w = x >> 3;
		r = x - (w << 3);

		//	Prepare mask
		m = (0x80 >> r);

		Data=fb[y][w];

		for(c = 0; c < 8; c++)
		{
			Temp <<= 1;
			if (Data&0x80)
			Temp  |= (1<<0);
			Data <<= 2;
		}

		Data |= m;

		  for(c = 0; c < 8; c++)
		  {
			  Temp <<= 2;
			  if (Data&0x80)
			  Temp  |= (1<<0);
			  Data <<= 1;
		  }

		 fb[y][w] |=Temp;
	}


	//===============================================================
	// Малюєм лінію
	//===============================================================
	void Put_Line(int16_t x1, int16_t y1, int16_t x2, int16_t y2)
	{
	    int16_t    dx, dy, i, e;
	    int16_t    incx, incy, inc1, inc2;
		int16_t    x, y;

	  	dx = x2 - x1;
	  	dy = y2 - y1;

	  	if(dx < 0) dx = -dx;
	  	if(dy < 0) dy = -dy;
	  	incx = 1;
	  	if(x2 < x1) incx = -1;
	  	incy = 1;
	  	if(y2 < y1) incy = -1;
	  	x=x1;
	  	y=y1;

	  	if (dx > dy)
	  	{
	  		Put_Pixel(x, y);
	  		e = 2*dy - dx;
	  		inc1 = 2 * ( dy -dx);
	  		inc2 = 2 * dy;
	  		for (i = 0; i < dx; i++)
	  		{
	  			if (e >= 0)
	  			{
	  				y += incy;
	  				e += inc1;
	  			}
	  			else
	  			{
	  				e += inc2;
	  			}
	  			x += incx;
	  			Put_Pixel(x, y);
	  		}
	  	}
	  	else
	  	{
	  		Put_Pixel(x, y);
	  		e = 2 * dx - dy;
	  		inc1 = 2 * (dx - dy);
	  		inc2 = 2 * dx;
	  		for(i = 0; i < dy; i++)
	  		{
	  			if (e >= 0)
	  			{
	  				x += incx;
	  				e += inc1;
	  			}
	  			else
	  			{
	  				e += inc2;
	  			}
	  			y += incy;
	  			Put_Pixel(x, y);
	  		}
	  	}
	}


	  //===============================================================
	  // Вимкнення монітора.
	  //===============================================================
	  void Monitor_Disable()
	  {
		  TIM_Cmd(TIM3, DISABLE);		// Вимкнення  таймера 3.
		  TIM_Cmd(TIM4, DISABLE);		// Вимкнення  таймера 4.
		  TIM_SetCounter(TIM3, 2100);	// Сигнал H_Sync низький рівень.
		  TIM_SetCounter(TIM4, 627);	// Сигнал V_Sync низький рівень.
	  }

	  //===============================================================
	  // Увімкнення монітора.
	  //===============================================================
	  void Monitor_Enable()
	  {
		  TIM_Cmd(TIM3, ENABLE);		// Вимкнення  таймера 3.
		  TIM_Cmd(TIM4, ENABLE);		// Вимкнення  таймера 4.
	  }









	void RCC_Configuration(void)
	{
		// Увімкнення тактування DMA, GPIOA, SPI, TIM3, TIM4.
		RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB, ENABLE);
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3 | RCC_APB1Periph_TIM4, ENABLE);
	}


	int main(void)
	{
		SystemInit();
		RCC_Configuration();
		vidInit();

		// Зафарбовування всього екрану фоновими пікселями сірого кольору, пікселі переднього плану матимуть чорний колір.
		Fill_With_Color(0, 300, 0, 50, Fon_White, Pixel_Black);		// (Y1,Y2,X1,X2, Колір фонових пікселів, колір пікселів переднього плану)

		while(1)
		{
	     	//           I-B-G-R-I-B-G-R-
			// Приклад кольрової вісімки лівий верхній кут.
			fb[0][0] = 0b0101010110101010;
			fb[1][0] = 0b1010110111010010;
			fb[2][0] = 0b1011110010010110;
			fb[3][0] = 0b1011011010011100;
			fb[4][0] = 0b1010011111011000;
			fb[5][0] = 0b1011011010011100;
			fb[6][0] = 0b1001111010110100;
			fb[7][0] = 0b1000111111110000;


			// 8 Кольорових смужок з низькою яскравістю.
			Fill_With_Color(24, 39, 5, 20, Fon_Black, Pixel_Black);		// (Y1,Y2,X1,X2, Колір фонових пікселів, колір пікселів переднього плану)
			Fill_With_Color(39, 54, 5, 20, Fon_Blue, Pixel_Blue);
			Fill_With_Color(54, 69, 5, 20, Fon_Red, Pixel_Red);
			Fill_With_Color(69, 84, 5, 20, Fon_Magenta, Pixel_Magenta);
			Fill_With_Color(84, 99, 5, 20, Fon_Green, Pixel_Green);
			Fill_With_Color(99, 114, 5, 20, Fon_Cyan, Pixel_Cyan);
			Fill_With_Color(114, 129, 5, 20, Fon_Yellow, Pixel_Yellow);
			Fill_With_Color(129, 144, 5, 20, Fon_White, Pixel_White);


			// 8 Кольорових смужок з максимальною яскравістю.
			Fill_With_Color(24, 39, 20, 35, Fon_Black, Pixel_Black);
			Fill_With_Color(39, 54, 20, 35, Fon_Bright_Blue, Pixel_Bright_Blue);
			Fill_With_Color(54, 69, 20, 35, Fon_Bright_Red, Pixel_Bright_Red);
			Fill_With_Color(69, 84, 20, 35, Fon_Bright_Magenta, Pixel_Bright_Magenta);
			Fill_With_Color(84, 99, 20, 35, Fon_Bright_Green, Pixel_Bright_Green);
			Fill_With_Color(99, 114, 20, 35, Fon_Bright_Cyan, Pixel_Bright_Cyan);
			Fill_With_Color(114, 129, 20, 35, Fon_Bright_Yellow, Pixel_Bright_Yellow);
			Fill_With_Color(129, 144, 20, 35, Fon_Bright_White, Pixel_Bright_White);


			// Задаємо колір фонових пікселів і колір пікселів переднього плану символа великого розміру.
			Fill_With_Color(160, 208, 4, 8, Fon_Black, Pixel_Bright_White);		// (Y1,Y2,X1,X2, Колір фонових пікселів, колір пікселів переднього плану)
			// Символ 32X48
			Big_Symbol(160, 4, 0); // Символ 0.

			Fill_With_Color(160, 208, 8, 12, Fon_Bright_Blue, Pixel_Bright_Yellow);
			Big_Symbol(160, 8, 1); // Символ 1.

			Fill_With_Color(160, 208, 12, 16, Fon_Bright_Red, Pixel_Bright_Cyan);
			Big_Symbol(160, 12, 2);

			Fill_With_Color(160, 208, 16, 20, Fon_Bright_Magenta, Pixel_Bright_Green);
			Big_Symbol(160, 16, 3);

			Fill_With_Color(160, 208, 20, 24, Fon_Bright_Green, Pixel_Bright_Magenta);
			Big_Symbol(160, 20, 4);

			Fill_With_Color(160, 208, 24, 28, Fon_Bright_Cyan, Pixel_Bright_Red);
			Big_Symbol(160, 24, 5);

			Fill_With_Color(160, 208, 28, 32, Fon_Bright_Yellow, Pixel_Bright_Blue);
			Big_Symbol(160, 28, 6);

			Fill_With_Color(160, 208, 32, 36, Fon_Bright_White, Pixel_Black);
			Big_Symbol(160, 32, 7);

			Big_Symbol(160, 36, 8);
			Big_Symbol(160, 40, 9);


			// Задаємо колір пікселів переднього плану, і колір фонових пікселів рядка символів середнього розміру.
			Fill_With_Color(220, 240, 0, 30, Fon_Black, Pixel_Bright_Yellow);		// (Y1,Y2,X1,X2, Колір фонових пікселів, колір пікселів переднього плану)
			// Символи 16X20
			Average_Symbol(220, 0, 0);
			Average_Symbol(220, 2, 1);
			Average_Symbol(220, 4, 2);
			Average_Symbol(220, 6, 3);
			Average_Symbol(220, 8, 4);
			Average_Symbol(220, 10,5);
			Average_Symbol(220, 12, 6);
			Average_Symbol(220, 14, 7);
			Average_Symbol(220, 16, 8);
			Average_Symbol(220, 18, 9);
			Average_Symbol(220, 20, 10);
			Average_Symbol(220, 22, 11);
			Average_Symbol(220, 24, 12);
			Average_Symbol(220, 26, 13);
			Average_Symbol(220, 28, 14);




			// Символи 8X8 з масива
			for (i=0;i<50;i++)
			{
				Small_Symbol(250, C, C);
				C++;
			}

			for (i=0;i<50;i++)
			{
				Small_Symbol(258, C, C);
				C++;
			}
			C=0;


			Put_Pixel(381, 37);				// Малюєм піксель (X1, Y1)
			Put_Pixel(383, 37);				// Малюєм піксель (X1, Y1)

			Put_Line(300, 10, 380, 100);	// Малюєм лінію (X1, Y1, X2, Y2)
			Put_Line(300, 90, 380, 49);		// Малюєм лінію (X1, Y1, X2, Y2)


			for (i=0;i<7200000;i++){}	// Затримка
			for (i=0;i<7200000;i++){}
			for (i=0;i<7200000;i++){}

//			Monitor_Disable();			// Вимкнення монітора.

//			for (i=0;i<7200000;i++){}	// Затримка
//			for (i=0;i<7200000;i++){}
//			for (i=0;i<7200000;i++){}

//			Monitor_Enable();			// Увімкнення монітора.

//			for (i=0;i<7200000;i++){}	// Затримка
//			for (i=0;i<7200000;i++){}
//			for (i=0;i<7200000;i++){}



		}
	}




















